// Mike Woodring
// Bear Canyon Consulting LLC
// http://www.bearcanyon.com
//
using System;
using System.Collections;
using System.Threading;
using System.Security;
using System.Security.Principal;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;
using BearCanyon;

class Client
{
    static void Main(string[] args)
    {
#if (true)
        // Take this path to test handling of a WindowsPrincipal.
        //
        AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
#else
        // Take this path to test handling of a GenericPrincipal.
        //
        Thread.CurrentPrincipal =
            new GenericPrincipal( new GenericIdentity("Moe"),
                                  new string[] {"Sex Symbols"} );
#endif

        ConfigureRemoting();
        Console.WriteLine("Client identity: {0}", Thread.CurrentPrincipal.Identity.Name);

        ICalc c = (ICalc)
            RemotingServices.Connect(typeof(ICalc), "http://localhost:999/calcsrv/calc");
        Console.WriteLine("2 + 3 = {0}", c.Add(2, 3));
    }

    static void ConfigureRemoting()
    {
#if (false)
        RemotingConfiguration.Configure(AppDomain.CurrentDomain.BaseDirectory + "client.exe.config");
#else
        // Programmatically do the same thing that's specified in the configuration file.
        //

        Hashtable emptyProps = new Hashtable();
        Hashtable props = new Hashtable();

        // Initialize the custom client sink (appears before the soap formatter in this sample).
        //
        props["date"] = DateTime.Now;
        IdentityClientSinkProvider clientSinks = new IdentityClientSinkProvider(props, emptyProps);

        // Initialize the soap formatter, connecting it to the tail end of the identify sink
        // that was initialized above.
        //
        clientSinks.Next = new SoapClientFormatterSinkProvider();

        // At this point, clientSinks -> IdentityClientSinkProvider -> SoapClientFormatterSinkProvider.
        // Reorder the above lines of code to produce an alternate configuration where a custom
        // sink follows, instead of precedes, a standard sink.

        // Initialize the server sink (for callbacks).
        //
        SoapServerFormatterSinkProvider serverSinks = new SoapServerFormatterSinkProvider();

        // Initialize the channel, passing in the client- & server-sinks setup above.
        //
        props = new Hashtable();
        props["port"] = 0;
        HttpChannel chan = new HttpChannel(props, clientSinks, serverSinks);
        ChannelServices.RegisterChannel(chan);

        // If you're using operator new to instantiate types, you'd also need to make the
        // appropriate calls to RemotingConfiguration.Register(WellKnown|Activated)ClientType.
#endif
    }
}