// IdentityClientSink implementation.
//
// Mike Woodring
// Bear Canyon Consulting LLC
// http://www.bearcanyon.com
//
using System;
using System.Collections;
using System.IO;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Messaging;
using System.Threading;
using System.Security.Principal;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace BearCanyon
{
    public class IdentityClientSinkProvider: IClientChannelSinkProvider 
    {
        private IClientChannelSinkProvider next = null;

        public IdentityClientSinkProvider () 
        {
        }

        public IdentityClientSinkProvider(IDictionary properties, ICollection providerData)
        {
            if( properties != null )
            {
                foreach( string key in properties.Keys )
                {
                    Console.WriteLine("{0} = {1}", key, properties[key]);
                }
            }
        }

        public IClientChannelSink CreateSink(IChannelSender channel, string url, object remoteChannelData)
        {
            return new IdentityClientSink(next.CreateSink(channel,url,remoteChannelData));
        }

        public IClientChannelSinkProvider Next
        {
            get { return next; }
            set { next = value; }
        }

    }

    public class IdentityClientSink : BaseChannelObjectWithProperties, IMessageSink, IClientChannelSink
    {	
        private IClientChannelSink _nextChnlSink;
        private IMessageSink _nextMsgSink;

        public IMessageCtrl AsyncProcessMessage( IMessage msg, IMessageSink replySink )
        {
            return _nextMsgSink.AsyncProcessMessage(msg,replySink);
        }

        public IMessage SyncProcessMessage( IMessage msg )
        {
            LogicalCallContext lcc = (LogicalCallContext) msg.Properties["__CallContext"];
            byte[] currentPrincipal = Util.CaptureIdentity();

            if( currentPrincipal != null )
            {
                lcc.SetData("identity", currentPrincipal);
            }

            return _nextMsgSink.SyncProcessMessage(msg);
        }

        public IdentityClientSink (object next) 
        {
            _nextMsgSink = next as IMessageSink;
            _nextChnlSink = next as IClientChannelSink;
        }


        public IMessageSink NextSink 
        {
            get 
            {
                return _nextMsgSink;
            }
        }

        public IClientChannelSink NextChannelSink { get { return _nextChnlSink;} }

        public void AsyncProcessRequest(IClientChannelSinkStack sinkStack, IMessage msg, ITransportHeaders headers, Stream stream) 
        {
            throw new Exception("Wrong sequence in config file");
        }

        public void AsyncProcessResponse(IClientResponseChannelSinkStack sinkStack, object state, ITransportHeaders headers, Stream stream)
        {
            throw new Exception("Wrong sequence in config file");
        }

        public Stream GetRequestStream(IMessage msg, ITransportHeaders headers)
        {
            throw new Exception("Wrong sequence in config file");
        }

        public void ProcessMessage(IMessage msg, ITransportHeaders requestHeaders, Stream requestStream, out ITransportHeaders responseHeaders, out Stream responseStream)
        {
            throw new Exception("Wrong sequence in config file");
        }
    }
}