// IdentityServerSink implementation.
//
// Mike Woodring
// Bear Canyon Consulting LLC
// http://www.bearcanyon.com
//
using System;
using System.Collections;
using System.IO;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Channels;
using System.Threading;
using System.Security.Principal;

namespace BearCanyon
{
    public class IdentityServerSinkProvider : IServerChannelSinkProvider 
    {
        private IServerChannelSinkProvider next = null;

        public IdentityServerSinkProvider()
        {
        }   

        public IdentityServerSinkProvider(IDictionary properties, ICollection providerData)
        {
        }

        public void GetChannelData (IChannelDataStore channelData)
        {
        }

        public IServerChannelSink CreateSink (IChannelReceiver channel)
        {
            IServerChannelSink nextSink = null;
            if (next != null) 
            {
                nextSink = next.CreateSink(channel);
            }
            return new IdentityServerSink(nextSink);
        }

        public IServerChannelSinkProvider Next
        {
            get { return next; }
            set { next = value; }
        }

    }

    public class IdentityServerSink : BaseChannelObjectWithProperties, IServerChannelSink, IChannelSinkBase
    {	
        private IServerChannelSink _next;

        public IdentityServerSink (IServerChannelSink next) 
        {
            _next = next;
        }

        public void AsyncProcessResponse ( IServerResponseChannelSinkStack sinkStack, object state, IMessage msg , ITransportHeaders headers , Stream stream ) 
        {
        }

        public Stream GetResponseStream ( IServerResponseChannelSinkStack sinkStack, object state, IMessage msg , ITransportHeaders headers ) 
        {
            return null;
        }
	
        public ServerProcessing ProcessMessage( IServerChannelSinkStack sinkStack, IMessage requestMsg, ITransportHeaders requestHeaders, Stream requestStream, out System.Runtime.Remoting.Messaging.IMessage responseMsg, out ITransportHeaders responseHeaders, out Stream responseStream)
        {
            if (_next != null) 
            {
                LogicalCallContext lcc = (LogicalCallContext) requestMsg.Properties["__CallContext"];
                IPrincipal originalPrincipal = null;

                if( lcc != null )
                {
                    byte[] principalData = lcc.GetData("identity") as byte[];

                    if( principalData != null )
                    {
                        IPrincipal originalCaller = Util.RetrieveIdentity(principalData);

                        if( originalCaller != null )
                        {
                            originalPrincipal = Thread.CurrentPrincipal;
                            Thread.CurrentPrincipal = originalCaller;
                        }
                    }
                }

                ServerProcessing spres =  _next.ProcessMessage (sinkStack,requestMsg, requestHeaders,requestStream,out responseMsg,out responseHeaders,out responseStream);

                if( originalPrincipal != null )
                {
                    // Restore identity before returning.
                    Thread.CurrentPrincipal = originalPrincipal;
                }

                return spres;
            } 
            else 
            {
                responseMsg=null;
                responseHeaders=null;
                responseStream=null;
                return new ServerProcessing();
            }
        }

        public IServerChannelSink NextChannelSink 
        {
            get {return _next;}
            set {_next = value;}
        }
    }
}