using System;
using System.IO;
using System.Threading;
using System.Security.Principal;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;
using System.Net;
using System.Collections;

namespace BearCanyon
{
    class Util
    {
        public static byte[] CaptureIdentity()
        {
            IPrincipal p = Thread.CurrentPrincipal;

            if( p is WindowsPrincipal )
            {
                // The WindowsPrincipal class demand-loads the table
                // of roles the first time IsInRole is called with
                // a non-null string.
                //
                p.IsInRole("");

                try
                {
                    // Compose the "AuthenticationType" string such that it
                    // reflects the actual setting, as well as this machine's
                    // name and IP address (primarily just to differentiate it from the
                    // stock authentication mechanism that was used on the client
                    // machine).  This also provides a record of the original
                    // machine name in the machine name substitution that is
                    // performed below for local gropu names.
                    //
                    // For example, if the AuthenticationType string of the calling
                    // thread's principal is "NTLM", and the local machine name
                    // is BEARTOOTH (with an IP address of 1.2.3.4), then the new
                    // AuthenticationType string will be "NTLM (BEARTOOTH:1.2.3.4)".
                    //
                    string machineName = Environment.MachineName;
                    string ipAddress = GetIPAddrForMachine(machineName);

                    // Construct serializable identity.
                    IIdentity id =
                        new GenericIdentity( p.Identity.Name,
                        string.Format("{0} ({1}:{2})", p.Identity.AuthenticationType, machineName, ipAddress));

                    // Retrieve roles.
                    FieldInfo fi = typeof(WindowsPrincipal).GetField("m_roles", BindingFlags.Instance | BindingFlags.NonPublic);
                
                    if( fi != null )
                    {
                        string[] roles = fi.GetValue(p) as string[];

                        if( roles == null )
                        {
                            // Perform an alternative check of the m_rolesTable Hashtable,
                            // which is used instead of the simple array if the
                            // calling thread is a member of a large number of NT groups
                            // (currently 23).
                            //
                            fi = typeof(WindowsPrincipal).GetField("m_rolesTable", BindingFlags.Instance | BindingFlags.NonPublic);
                            Hashtable roleTable = fi.GetValue(p) as Hashtable;

                            if( roleTable != null )
                            {
                                roles = new string[roleTable.Count];

                                IDictionaryEnumerator enumerator = roleTable.GetEnumerator();
                                int n = 0;

                                while( enumerator.MoveNext() )
                                {
                                    roles[n++] = (string)enumerator.Value;
                                }
                            }
                        }

                        if( roles != null )
                        {
                            // For any role we find that is of the form
                            // <local machine name>\<role>, replace
                            // <local machine name> with ".".  For example,
                            // BEARTOOTH\Administrators will become
                            // .\Administrators.
                            //
                            for( int n = 0; n < roles.Length; n++ )
                            {
                                if( (roles[n] != null) && (roles[n] != string.Empty) )
                                {
                                    roles[n] = roles[n].Replace(machineName + "\\", ".\\");
                                }
                            }

                            p = new GenericPrincipal(id, roles);
                        }
                    }
                }
                catch
                {
                }
            }
            else if( p is GenericPrincipal )
            {
                // This type of principal can be shipped
                // across as-is.
            }
            else
            {
                // Unknown (custom) principal type isn't handled.
            }

            MemoryStream stm = new MemoryStream(256);
            BinaryFormatter bf = new BinaryFormatter();
            long cb = 0;

            try
            {
                bf.Serialize(stm, p);
                cb = stm.Position;
            }
            catch
            {
            }

            stm.Close();

            byte[] principalData = null;

            if( cb > 0 )
            {
                byte[] buf = stm.GetBuffer();
                principalData = new byte[cb];
                Array.Copy(buf, 0, principalData, 0, (int)cb);
            }

            return(principalData);
        }

        public static IPrincipal RetrieveIdentity( byte[] buf )
        {
            MemoryStream stm = new MemoryStream(buf);
            BinaryFormatter bf = new BinaryFormatter();
            IPrincipal p = null;

            try
            {
                p = (IPrincipal)bf.Deserialize(stm);
            }
            catch
            {
            }

            stm.Close();
            return(p);
        }

        private static string GetIPAddrForMachine( string machineName )
        {
            try
            {
                IPHostEntry ipRec = Dns.GetHostByName(machineName);
                IPAddress ipAddr = ipRec.AddressList[0];
                return(ipAddr.ToString());
            }
            catch
            {
                return("");
            }
        }
    }
}